@@ -704,7 +704,7 @@ applet_about_dialog_show (NMApplet *applet)
 #endif
 	gtk_show_about_dialog (NULL,
 	                       "version", VERSION,
-	                       "copyright", _("Copyright \xc2\xa9 2004-2010 Red Hat, Inc.\n"
+	                       "copyright", _("Copyright \xc2\xa9 2004-2011 Red Hat, Inc.\n"
 	                                      "Copyright \xc2\xa9 2005-2008 Novell, Inc.\n"
 	                                      "and many other community contributors and translators"),
 	                       "comments", _("Notification area applet for managing your network devices and connections."),
@@ -905,7 +905,7 @@ applet_mobile_pin_dialog_new (const char *title,
 
 	builder = gtk_builder_new ();
 
-	if (!gtk_builder_add_from_file (builder, "applet.ui", &error)) {
+	if (!gtk_builder_add_from_file (builder, UIDIR "/gsm-unlock.ui", &error)) {
 		g_warning ("Couldn't load builder file: %s", error->message);
 		g_error_free (error);
 		g_object_unref (builder);
