@@ -19,7 +19,7 @@
  * with this program; if not, write to the Free Software Foundation, Inc.,
  * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
- * (C) Copyright 2007 - 2010 Red Hat, Inc.
+ * (C) Copyright 2007 - 2011 Red Hat, Inc.
  * (C) Copyright 2007 - 2008 Novell, Inc.
  */
 
@@ -115,7 +115,7 @@ ui_to_setting (NMConnectionEditor *editor)
 	s_con = NM_SETTING_CONNECTION (nm_connection_get_setting (editor->connection, NM_TYPE_SETTING_CONNECTION));
 	g_assert (s_con);
 
-	widget = glade_xml_get_widget (editor->xml, "connection_name");
+	widget = GTK_WIDGET (gtk_builder_get_object (editor->builder, "connection_name"));
 	name = gtk_entry_get_text (GTK_ENTRY (widget));
 
 	g_object_set (G_OBJECT (s_con), NM_SETTING_CONNECTION_ID, name, NULL);
@@ -124,7 +124,7 @@ ui_to_setting (NMConnectionEditor *editor)
 	if (!name || !strlen (name))
 		return FALSE;
 
-	widget = glade_xml_get_widget (editor->xml, "connection_autoconnect");
+	widget = GTK_WIDGET (gtk_builder_get_object (editor->builder, "connection_autoconnect"));
 	autoconnect = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget));
 	g_object_set (G_OBJECT (s_con), NM_SETTING_CONNECTION_AUTOCONNECT, autoconnect, NULL);
 
@@ -182,16 +182,16 @@ update_sensitivity (NMConnectionEditor *editor)
 	/* Cancel button is always sensitive */
 	gtk_widget_set_sensitive (GTK_WIDGET (editor->cancel_button), TRUE);
 
-	widget = glade_xml_get_widget (editor->xml, "connection_name_label");
+	widget = GTK_WIDGET (gtk_builder_get_object (editor->builder, "connection_name_label"));
 	gtk_widget_set_sensitive (widget, sensitive);
 
-	widget = glade_xml_get_widget (editor->xml, "connection_name");
+	widget = GTK_WIDGET (gtk_builder_get_object (editor->builder, "connection_name"));
 	gtk_widget_set_sensitive (widget, sensitive);
 
-	widget = glade_xml_get_widget (editor->xml, "connection_autoconnect");
+	widget = GTK_WIDGET (gtk_builder_get_object (editor->builder, "connection_autoconnect"));
 	gtk_widget_set_sensitive (widget, sensitive);
 
-	widget = glade_xml_get_widget (editor->xml, "connection_name");
+	widget = GTK_WIDGET (gtk_builder_get_object (editor->builder, "connection_name"));
 	gtk_widget_set_sensitive (widget, sensitive);
 
 	for (iter = editor->pages; iter; iter = g_slist_next (iter)) {
@@ -276,37 +276,32 @@ static void
 nm_connection_editor_init (NMConnectionEditor *editor)
 {
 	GtkWidget *dialog;
+	GError *error = NULL;
+	const char *objects[] = { "nm-connection-editor", NULL };
+
+	editor->builder = gtk_builder_new ();
+
+	if (!gtk_builder_add_objects_from_file (editor->builder,
+	                                        UIDIR "/nm-connection-editor.ui",
+	                                        (char **) objects,
+	                                        &error)) {
+		g_warning ("Couldn't load builder file " UIDIR "/nm-connection-editor.ui: %s", error->message);
+		g_error_free (error);
 
-	/* Yes, we mean applet.glade, not nm-connection-editor.glade. The wireless security bits
-	   are taken from applet.glade. */
-	if (!g_file_test (GLADEDIR "/applet.glade", G_FILE_TEST_EXISTS)) {
 		dialog = gtk_message_dialog_new (NULL, 0,
 		                                 GTK_MESSAGE_ERROR,
 		                                 GTK_BUTTONS_OK,
 		                                 "%s",
-		                                 _("The connection editor could not find some required resources (the NetworkManager applet glade file was not found)."));
+		                                 _("The connection editor could not find some required resources (the .ui file was not found)."));
 		gtk_dialog_run (GTK_DIALOG (dialog));
 		gtk_widget_destroy (dialog);
 		gtk_main_quit ();
 		return;
 	}
 
-	editor->xml = glade_xml_new (GLADEDIR "/nm-connection-editor.glade", "nm-connection-editor", NULL);
-	if (!editor->xml) {
-		dialog = gtk_message_dialog_new (NULL, 0,
-		                                 GTK_MESSAGE_ERROR,
-		                                 GTK_BUTTONS_OK,
-		                                 "%s",
-		                                 _("The connection editor could not find some required resources (the glade file was not found)."));
-		gtk_dialog_run (GTK_DIALOG (dialog));
-		gtk_widget_destroy (dialog);
-		gtk_main_quit ();
-		return;
-	}
-
-	editor->window = glade_xml_get_widget (editor->xml, "nm-connection-editor");
-	editor->cancel_button = glade_xml_get_widget (editor->xml, "cancel_button");
-	editor->system_checkbutton = glade_xml_get_widget (editor->xml, "system_checkbutton");
+	editor->window = GTK_WIDGET (gtk_builder_get_object (editor->builder, "nm-connection-editor"));
+	editor->cancel_button = GTK_WIDGET (gtk_builder_get_object (editor->builder, "cancel_button"));
+	editor->system_checkbutton = GTK_WIDGET (gtk_builder_get_object (editor->builder, "system_checkbutton"));
 }
 
 static void
@@ -355,9 +350,9 @@ dispose (GObject *object)
 		gtk_widget_destroy (editor->window);
 		editor->window = NULL;
 	}
-	if (editor->xml) {
-		g_object_unref (editor->xml);
-		editor->xml = NULL;
+	if (editor->builder) {
+		g_object_unref (editor->builder);
+		editor->builder = NULL;
 	}
 
 	G_OBJECT_CLASS (nm_connection_editor_parent_class)->dispose (object);
@@ -430,7 +425,7 @@ nm_connection_editor_new (NMConnection *connection,
 	                  G_CALLBACK (ok_button_actionable_cb), editor);
 	g_signal_connect (editor->ok_button, "authorized",
 	                  G_CALLBACK (ok_button_actionable_cb), editor);
-	hbox = glade_xml_get_widget (editor->xml, "action_area_hbox");
+	hbox = GTK_WIDGET (gtk_builder_get_object (editor->builder, "action_area_hbox"));
 	gtk_box_pack_end (GTK_BOX (hbox), editor->ok_button, TRUE, TRUE, 0);
 	gtk_widget_show_all (editor->ok_button);
 
@@ -477,8 +472,8 @@ populate_connection_ui (NMConnectionEditor *editor)
 	GtkWidget *name;
 	GtkWidget *autoconnect;
 
-	name = glade_xml_get_widget (editor->xml, "connection_name");
-	autoconnect = glade_xml_get_widget (editor->xml, "connection_autoconnect");
+	name = GTK_WIDGET (gtk_builder_get_object (editor->builder, "connection_name"));
+	autoconnect = GTK_WIDGET (gtk_builder_get_object (editor->builder, "connection_autoconnect"));
 
 	s_con = NM_SETTING_CONNECTION (nm_connection_get_setting (editor->connection, NM_TYPE_SETTING_CONNECTION));
 	if (s_con) {
@@ -520,9 +515,11 @@ idle_validate (gpointer user_data)
 static void
 recheck_initialization (NMConnectionEditor *editor)
 {
-	if (!editor_is_initialized (editor))
+	if (!editor_is_initialized (editor) || editor->init_run)
 		return;
 
+	editor->init_run = TRUE;
+
 	populate_connection_ui (editor);
 
 	/* When everything is initialized, re-present the window to ensure it's on top */
@@ -535,11 +532,11 @@ recheck_initialization (NMConnectionEditor *editor)
 }
 
 static void
-page_initialized (CEPage *page, gpointer unused, GError *error, gpointer user_data)
+page_initialized (CEPage *page, GError *error, gpointer user_data)
 {
 	NMConnectionEditor *editor = NM_CONNECTION_EDITOR (user_data);
 	GtkWidget *widget;
-	GtkWidget *notebook;
+	GtkNotebook *notebook;
 	GtkWidget *label;
 
 	if (error) {
@@ -549,10 +546,11 @@ page_initialized (CEPage *page, gpointer unused, GError *error, gpointer user_da
 	}
 
 	/* Add the page to the UI */
-	notebook = glade_xml_get_widget (editor->xml, "notebook");
+	notebook = GTK_NOTEBOOK (gtk_builder_get_object (editor->builder, "notebook"));
 	label = gtk_label_new (ce_page_get_title (page));
 	widget = ce_page_get_page (page);
-	gtk_notebook_append_page (GTK_NOTEBOOK (notebook), widget, label);
+	gtk_widget_unparent (widget);
+	gtk_notebook_append_page (notebook, widget, label);
 
 	/* Move the page from the initializing list to the main page list */
 	editor->initializing_pages = g_slist_remove (editor->initializing_pages, page);
@@ -627,25 +625,6 @@ get_secrets_for_page (NMConnectionEditor *self,
 {
 	GetSecretsInfo *info;
 
-	if (!setting_name) {
-		/* page doesn't need any secrets */
-		ce_page_complete_init (page, NULL, NULL, NULL);
-		return;
-	}
-
-	/* Try to get secrets from ->orig_connection, because it's the one that
-	 * implements NMSettingsConnectionInterface and can respond to requests for
-	 * its secrets.  ->connection is a plain NMConnection copy of ->orig_connection
-	 * which is the connection that's actually changed when the user clicks stuff.
-	 * When creating importing or creating new connections though, ->orig_connection
-	 * is an NMConnection because it hasn't been exported over D-Bus yet, so we
-	 * can't ask it for secrets, because it doesn't implement NMSettingsConnectionInterface.
-	 */
-	if (!NM_IS_SETTINGS_CONNECTION_INTERFACE (self->orig_connection)) {
-		ce_page_complete_init (page, setting_name, NULL, NULL);
-		return;
-	}
-
 	info = g_malloc0 (sizeof (GetSecretsInfo));
 	info->self = self;
 	info->page = page;
@@ -673,6 +652,8 @@ get_secrets_for_page (NMConnectionEditor *self,
 	}
 }
 
+#define SECRETS_TAG "secrets-setting-name"
+
 static gboolean
 add_page (NMConnectionEditor *editor,
           CEPageNewFunc func,
@@ -687,19 +668,17 @@ add_page (NMConnectionEditor *editor,
 	g_return_val_if_fail (connection != NULL, FALSE);
 
 	page = (*func) (connection, GTK_WINDOW (editor->window), &secrets_setting_name, error);
-	if (!page)
-		return FALSE;
+	if (page) {
+		g_object_set_data_full (G_OBJECT (page),
+			                    SECRETS_TAG,
+			                    g_strdup (secrets_setting_name),
+			                    g_free);
 
-	editor->initializing_pages = g_slist_prepend (editor->initializing_pages, page);
-	g_signal_connect (page, "changed", G_CALLBACK (page_changed), editor);
-	g_signal_connect (page, "initialized", G_CALLBACK (page_initialized), editor);
-
-	/* Request any secrets the page might require; or if it doesn't want any,
-	 * let the page initialize.
-	 */
-	get_secrets_for_page (editor, page, secrets_setting_name);
-
-	return TRUE;
+		editor->initializing_pages = g_slist_append (editor->initializing_pages, page);
+		g_signal_connect (page, "changed", G_CALLBACK (page_changed), editor);
+		g_signal_connect (page, "initialized", G_CALLBACK (page_initialized), editor);
+	}
+	return !!page;
 }
 
 static gboolean
@@ -710,6 +689,7 @@ nm_connection_editor_set_connection (NMConnectionEditor *editor,
 	NMSettingConnection *s_con;
 	const char *connection_type;
 	gboolean success = FALSE;
+	GSList *iter, *copy;
 
 	g_return_val_if_fail (NM_IS_CONNECTION_EDITOR (editor), FALSE);
 	g_return_val_if_fail (NM_IS_CONNECTION (orig_connection), FALSE);
@@ -772,6 +752,38 @@ nm_connection_editor_set_connection (NMConnectionEditor *editor,
 		g_warning ("Unhandled setting type '%s'", connection_type);
 	}
 
+	/* After all pages are created, then kick off secrets requests that any
+	 * the pages may need to make; if they don't need any secrets, then let
+	 * them finish initialization.  The list might get modified during the loop
+	 * which is why copy the list here.
+	 */
+	copy = g_slist_copy (editor->initializing_pages);
+	for (iter = copy; iter; iter = g_slist_next (iter)) {
+		CEPage *page = CE_PAGE (iter->data);
+		const char *setting_name = g_object_get_data (G_OBJECT (page), SECRETS_TAG);
+
+		if (!setting_name) {
+			/* page doesn't need any secrets */
+			ce_page_complete_init (page, NULL, NULL, NULL);
+		} else if (!NM_IS_SETTINGS_CONNECTION_INTERFACE (editor->orig_connection)) {
+			/* We want to get secrets using ->orig_connection, since that's the one
+			 * that implements NMSettingsConnectionInterface and thus can actually
+			 * handle secrets requests.  ->connection is a plain NMConnection copy
+			 * of ->orig_connection which is the thing that gets changed when users
+			 * modify anything.  But when creating or importing, ->orig_connection
+			 * will be an NMConnection since the new connection hasn't been added to
+			 * any settings service yet.  So basically, skip requesting secrets if
+			 * the connection can't handle a secrets request.
+			 */
+			ce_page_complete_init (page, setting_name, NULL, NULL);
+		} else {
+			/* Page wants secrets, get them */
+			get_secrets_for_page (editor, page, setting_name);
+		}
+		g_object_set_data (G_OBJECT (page), SECRETS_TAG, NULL);
+	}
+	g_slist_free (copy);
+
 	/* set the UI */
 	recheck_initialization (editor);
 	success = TRUE;
