@@ -107,13 +107,13 @@ static void
 ip4_private_init (CEPageIP4 *self, NMConnection *connection)
 {
 	CEPageIP4Private *priv = CE_PAGE_IP4_GET_PRIVATE (self);
-	GladeXML *xml;
+	GtkBuilder *builder;
 	GtkTreeIter iter;
 	NMSettingConnection *s_con;
 	const char *connection_type;
 	char *str_auto = NULL, *str_auto_only = NULL;
 
-	xml = CE_PAGE (self)->xml;
+	builder = CE_PAGE (self)->builder;
 
 	s_con = NM_SETTING_CONNECTION (nm_connection_get_setting (connection, NM_TYPE_SETTING_CONNECTION));
 	g_assert (s_con);
@@ -137,7 +137,7 @@ ip4_private_init (CEPageIP4 *self, NMConnection *connection)
 		str_auto_only = _("Automatic (DHCP) addresses only");
 	}
 
-	priv->method = GTK_COMBO_BOX (glade_xml_get_widget (xml, "ip4_method"));
+	priv->method = GTK_COMBO_BOX (GTK_WIDGET (gtk_builder_get_object (builder, "ip4_method")));
 
 	priv->method_store = gtk_list_store_new (2, G_TYPE_STRING, G_TYPE_UINT);
 
@@ -193,19 +193,19 @@ ip4_private_init (CEPageIP4 *self, NMConnection *connection)
 	}
 	gtk_combo_box_set_model (priv->method, GTK_TREE_MODEL (priv->method_store));
 
-	priv->addr_label = glade_xml_get_widget (xml, "ip4_addr_label");
-	priv->addr_add = GTK_BUTTON (glade_xml_get_widget (xml, "ip4_addr_add_button"));
-	priv->addr_delete = GTK_BUTTON (glade_xml_get_widget (xml, "ip4_addr_delete_button"));
-	priv->addr_list = GTK_TREE_VIEW (glade_xml_get_widget (xml, "ip4_addresses"));
+	priv->addr_label = GTK_WIDGET (gtk_builder_get_object (builder, "ip4_addr_label"));
+	priv->addr_add = GTK_BUTTON (GTK_WIDGET (gtk_builder_get_object (builder, "ip4_addr_add_button")));
+	priv->addr_delete = GTK_BUTTON (GTK_WIDGET (gtk_builder_get_object (builder, "ip4_addr_delete_button")));
+	priv->addr_list = GTK_TREE_VIEW (GTK_WIDGET (gtk_builder_get_object (builder, "ip4_addresses")));
 
-	priv->dns_servers_label = glade_xml_get_widget (xml, "ip4_dns_servers_label");
-	priv->dns_servers = GTK_ENTRY (glade_xml_get_widget (xml, "ip4_dns_servers_entry"));
+	priv->dns_servers_label = GTK_WIDGET (gtk_builder_get_object (builder, "ip4_dns_servers_label"));
+	priv->dns_servers = GTK_ENTRY (GTK_WIDGET (gtk_builder_get_object (builder, "ip4_dns_servers_entry")));
 
-	priv->dns_searches_label = glade_xml_get_widget (xml, "ip4_dns_searches_label");
-	priv->dns_searches = GTK_ENTRY (glade_xml_get_widget (xml, "ip4_dns_searches_entry"));
+	priv->dns_searches_label = GTK_WIDGET (gtk_builder_get_object (builder, "ip4_dns_searches_label"));
+	priv->dns_searches = GTK_ENTRY (GTK_WIDGET (gtk_builder_get_object (builder, "ip4_dns_searches_entry")));
 
-	priv->dhcp_client_id_label = glade_xml_get_widget (xml, "ip4_dhcp_client_id_label");
-	priv->dhcp_client_id = GTK_ENTRY (glade_xml_get_widget (xml, "ip4_dhcp_client_id_entry"));
+	priv->dhcp_client_id_label = GTK_WIDGET (gtk_builder_get_object (builder, "ip4_dhcp_client_id_label"));
+	priv->dhcp_client_id = GTK_ENTRY (GTK_WIDGET (gtk_builder_get_object (builder, "ip4_dhcp_client_id_entry")));
 
 	/* Hide DHCP stuff if it'll never be used for a particular method */
 	if (   priv->connection_type == NM_TYPE_SETTING_VPN
@@ -216,7 +216,7 @@ ip4_private_init (CEPageIP4 *self, NMConnection *connection)
 		gtk_widget_hide (GTK_WIDGET (priv->dhcp_client_id));
 	}
 
-	priv->ip4_required = GTK_CHECK_BUTTON (glade_xml_get_widget (xml, "ip4_required_checkbutton"));
+	priv->ip4_required = GTK_CHECK_BUTTON (GTK_WIDGET (gtk_builder_get_object (builder, "ip4_required_checkbutton")));
 	/* Hide IP4-require button if it'll never be used for a particular method */
 	if (   priv->connection_type == NM_TYPE_SETTING_VPN
 	    || priv->connection_type == NM_TYPE_SETTING_GSM
@@ -224,7 +224,7 @@ ip4_private_init (CEPageIP4 *self, NMConnection *connection)
 	    || priv->connection_type == NM_TYPE_SETTING_PPPOE)
 		gtk_widget_hide (GTK_WIDGET (priv->ip4_required));
 
-	priv->routes_button = GTK_BUTTON (glade_xml_get_widget (xml, "ip4_routes_button"));
+	priv->routes_button = GTK_BUTTON (GTK_WIDGET (gtk_builder_get_object (builder, "ip4_routes_button")));
 }
 
 static void
@@ -601,7 +601,9 @@ ip_address_filter_cb (GtkEntry *   entry,
 	CEPageIP4Private *priv = CE_PAGE_IP4_GET_PRIVATE (self);
 	GtkEditable *editable = GTK_EDITABLE (entry);
 	int i, count = 0;
-	gchar *result = g_new0 (gchar, length);
+	gchar *result;
+
+	result = g_malloc0 (length + 1);
 
 	for (i = 0; i < length; i++) {
 		if ((text[i] >= '0' && text[i] <= '9') || (text[i] == '.'))
@@ -805,32 +807,19 @@ ce_page_ip4_new (NMConnection *connection,
 {
 	CEPageIP4 *self;
 	CEPageIP4Private *priv;
-	CEPage *parent;
 	NMSettingConnection *s_con;
 
-	self = CE_PAGE_IP4 (g_object_new (CE_TYPE_PAGE_IP4,
-	                                  CE_PAGE_CONNECTION, connection,
-	                                  CE_PAGE_PARENT_WINDOW, parent_window,
-	                                  NULL));
-	parent = CE_PAGE (self);
-
-	parent->xml = glade_xml_new (GLADEDIR "/ce-page-ip4.glade", "IP4Page", NULL);
-	if (!parent->xml) {
-		g_set_error (error, 0, 0, "%s", _("Could not load IPv4 user interface."));
-		g_object_unref (self);
+	self = CE_PAGE_IP4 (ce_page_new (CE_TYPE_PAGE_IP4,
+	                                 connection,
+	                                 parent_window,
+	                                 UIDIR "/ce-page-ip4.ui",
+	                                 "IP4Page",
+	                                 _("IPv4 Settings")));
+	if (!self) {
+		g_set_error_literal (error, 0, 0, _("Could not load IPv4 user interface."));
 		return NULL;
 	}
 
-	parent->page = glade_xml_get_widget (parent->xml, "IP4Page");
-	if (!parent->page) {
-		g_set_error (error, 0, 0, "%s", _("Could not load IPv4 user interface."));
-		g_object_unref (self);
-		return NULL;
-	}
-	g_object_ref_sink (parent->page);
-
-	parent->title = g_strdup (_("IPv4 Settings"));
-
 	ip4_private_init (self, connection);
 	priv = CE_PAGE_IP4_GET_PRIVATE (self);
 
