@@ -17,7 +17,7 @@
  * with this program; if not, write to the Free Software Foundation, Inc.,
  * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
- * (C) Copyright 2008 Red Hat, Inc.
+ * (C) Copyright 2008 - 2010 Red Hat, Inc.
  */
 
 #include <string.h>
@@ -33,6 +33,8 @@
 
 #include <nm-setting-connection.h>
 #include <nm-setting-ip6-config.h>
+#include <nm-setting-wired.h>
+#include <nm-setting-wireless.h>
 #include <nm-setting-gsm.h>
 #include <nm-setting-cdma.h>
 #include <nm-setting-pppoe.h>
@@ -48,7 +50,8 @@ G_DEFINE_TYPE (CEPageIP6, ce_page_ip6, CE_TYPE_PAGE)
 
 #define COL_ADDRESS 0
 #define COL_PREFIX 1
-#define COL_LAST COL_PREFIX
+#define COL_GATEWAY 2
+#define COL_LAST COL_GATEWAY
 
 typedef struct {
 	NMSettingIP6Config *setting;
@@ -73,10 +76,17 @@ typedef struct {
 	GtkWidget *dns_searches_label;
 	GtkEntry *dns_searches;
 
+	/* Routes */
 	GtkButton *routes_button;
 
+	/* IPv6 required */
+	GtkCheckButton *ip6_required;
+
 	GtkWindowGroup *window_group;
 	gboolean window_added;
+
+	/* Cached tree view entry for editing-canceled */
+	char *last_edited;
 } CEPageIP6Private;
 
 #define METHOD_COL_NAME 0
@@ -86,22 +96,23 @@ typedef struct {
 #define IP6_METHOD_IGNORE          0
 #define IP6_METHOD_AUTO            1
 #define IP6_METHOD_AUTO_ADDRESSES  2
-#define IP6_METHOD_MANUAL          3
-#define IP6_METHOD_LINK_LOCAL      4
-#define IP6_METHOD_SHARED          5
+#define IP6_METHOD_AUTO_DHCP_ONLY  3
+#define IP6_METHOD_MANUAL          4
+#define IP6_METHOD_LINK_LOCAL      5
+#define IP6_METHOD_SHARED          6
 
 static void
 ip6_private_init (CEPageIP6 *self, NMConnection *connection)
 {
 	CEPageIP6Private *priv = CE_PAGE_IP6_GET_PRIVATE (self);
-	GladeXML *xml;
+	GtkBuilder *builder;
 	GtkTreeIter iter;
 	NMSettingConnection *s_con;
 	const char *connection_type;
 	char *str_auto = NULL, *str_auto_only = NULL;
 	GList *cells;
 
-	xml = CE_PAGE (self)->xml;
+	builder = CE_PAGE (self)->builder;
 
 	s_con = NM_SETTING_CONNECTION (nm_connection_get_setting (connection, NM_TYPE_SETTING_CONNECTION));
 	g_assert (s_con);
@@ -125,7 +136,7 @@ ip6_private_init (CEPageIP6 *self, NMConnection *connection)
 		str_auto_only = _("Automatic, addresses only");
 	}
 
-	priv->method = GTK_COMBO_BOX (glade_xml_get_widget (xml, "ip6_method"));
+	priv->method = GTK_COMBO_BOX (GTK_WIDGET (gtk_builder_get_object (builder, "ip6_method")));
 	cells = gtk_cell_layout_get_cells (GTK_CELL_LAYOUT (priv->method));
 	gtk_cell_layout_add_attribute (GTK_CELL_LAYOUT (priv->method), cells->data,
 								   "sensitive", METHOD_COL_ENABLED);
@@ -153,6 +164,17 @@ ip6_private_init (CEPageIP6 *self, NMConnection *connection)
 						METHOD_COL_ENABLED, TRUE,
 	                    -1);
 
+	/* DHCP only used on wifi and wired for now */
+	if (   priv->connection_type == NM_TYPE_SETTING_WIRED
+	    || priv->connection_type == NM_TYPE_SETTING_WIRELESS) {
+		gtk_list_store_append (priv->method_store, &iter);
+		gtk_list_store_set (priv->method_store, &iter,
+		                    METHOD_COL_NAME, _("Automatic, DHCP only"),
+		                    METHOD_COL_NUM, IP6_METHOD_AUTO_DHCP_ONLY,
+							METHOD_COL_ENABLED, TRUE,
+		                    -1);
+	}
+
 	/* Manual is pointless for Mobile Broadband */
 	if (   priv->connection_type != NM_TYPE_SETTING_GSM
 	    && priv->connection_type != NM_TYPE_SETTING_CDMA
@@ -187,18 +209,26 @@ ip6_private_init (CEPageIP6 *self, NMConnection *connection)
 
 	gtk_combo_box_set_model (priv->method, GTK_TREE_MODEL (priv->method_store));
 
-	priv->addr_label = glade_xml_get_widget (xml, "ip6_addr_label");
-	priv->addr_add = GTK_BUTTON (glade_xml_get_widget (xml, "ip6_addr_add_button"));
-	priv->addr_delete = GTK_BUTTON (glade_xml_get_widget (xml, "ip6_addr_delete_button"));
-	priv->addr_list = GTK_TREE_VIEW (glade_xml_get_widget (xml, "ip6_addresses"));
+	priv->addr_label = GTK_WIDGET (gtk_builder_get_object (builder, "ip6_addr_label"));
+	priv->addr_add = GTK_BUTTON (GTK_WIDGET (gtk_builder_get_object (builder, "ip6_addr_add_button")));
+	priv->addr_delete = GTK_BUTTON (GTK_WIDGET (gtk_builder_get_object (builder, "ip6_addr_delete_button")));
+	priv->addr_list = GTK_TREE_VIEW (GTK_WIDGET (gtk_builder_get_object (builder, "ip6_addresses")));
 
-	priv->dns_servers_label = glade_xml_get_widget (xml, "ip6_dns_servers_label");
-	priv->dns_servers = GTK_ENTRY (glade_xml_get_widget (xml, "ip6_dns_servers_entry"));
+	priv->dns_servers_label = GTK_WIDGET (gtk_builder_get_object (builder, "ip6_dns_servers_label"));
+	priv->dns_servers = GTK_ENTRY (GTK_WIDGET (gtk_builder_get_object (builder, "ip6_dns_servers_entry")));
 
-	priv->dns_searches_label = glade_xml_get_widget (xml, "ip6_dns_searches_label");
-	priv->dns_searches = GTK_ENTRY (glade_xml_get_widget (xml, "ip6_dns_searches_entry"));
+	priv->dns_searches_label = GTK_WIDGET (gtk_builder_get_object (builder, "ip6_dns_searches_label"));
+	priv->dns_searches = GTK_ENTRY (GTK_WIDGET (gtk_builder_get_object (builder, "ip6_dns_searches_entry")));
 
-	priv->routes_button = GTK_BUTTON (glade_xml_get_widget (xml, "ip6_routes_button"));
+	priv->ip6_required = GTK_CHECK_BUTTON (GTK_WIDGET (gtk_builder_get_object (builder, "ip6_required_checkbutton")));
+	/* Hide IP6-require button if it'll never be used for a particular method */
+	if (   priv->connection_type == NM_TYPE_SETTING_VPN
+	    || priv->connection_type == NM_TYPE_SETTING_GSM
+	    || priv->connection_type == NM_TYPE_SETTING_CDMA
+	    || priv->connection_type == NM_TYPE_SETTING_PPPOE)
+		gtk_widget_hide (GTK_WIDGET (priv->ip6_required));
+
+	priv->routes_button = GTK_BUTTON (GTK_WIDGET (gtk_builder_get_object (builder, "ip6_routes_button")));
 }
 
 static void
@@ -209,6 +239,7 @@ method_changed (GtkComboBox *combo, gpointer user_data)
 	gboolean addr_enabled = FALSE;
 	gboolean dns_enabled = FALSE;
 	gboolean routes_enabled = FALSE;
+	gboolean ip6_required_enabled = TRUE;
 	GtkTreeIter iter;
 
 	if (gtk_combo_box_get_active_iter (priv->method, &iter)) {
@@ -225,9 +256,16 @@ method_changed (GtkComboBox *combo, gpointer user_data)
 		addr_enabled = FALSE;
 		dns_enabled = routes_enabled = TRUE;
 		break;
+	case IP6_METHOD_AUTO_DHCP_ONLY:
+		addr_enabled = FALSE;
+		routes_enabled = TRUE;
+		break;
 	case IP6_METHOD_MANUAL:
 		addr_enabled = dns_enabled = routes_enabled = TRUE;
 		break;
+	case IP6_METHOD_IGNORE:
+		ip6_required_enabled = FALSE;
+		break;
 	default:
 		break;
 	}
@@ -253,6 +291,8 @@ method_changed (GtkComboBox *combo, gpointer user_data)
 	if (!dns_enabled)
 		gtk_entry_set_text (priv->dns_searches, "");
 
+	gtk_widget_set_sensitive (GTK_WIDGET (priv->ip6_required), ip6_required_enabled);
+
 	gtk_widget_set_sensitive (GTK_WIDGET (priv->routes_button), routes_enabled);
 
 	ce_page_changed (CE_PAGE (user_data));
@@ -296,6 +336,8 @@ populate_ui (CEPageIP6 *self)
 	if (str_method) {
 		if (!strcmp (str_method, NM_SETTING_IP6_CONFIG_METHOD_AUTO))
 			method = IP6_METHOD_AUTO;
+		if (!strcmp (str_method, NM_SETTING_IP6_CONFIG_METHOD_DHCP))
+			method = IP6_METHOD_AUTO_DHCP_ONLY;
 		else if (!strcmp (str_method, NM_SETTING_IP6_CONFIG_METHOD_LINK_LOCAL))
 			method = IP6_METHOD_LINK_LOCAL;
 		else if (!strcmp (str_method, NM_SETTING_IP6_CONFIG_METHOD_MANUAL))
@@ -312,7 +354,7 @@ populate_ui (CEPageIP6 *self)
 	gtk_tree_model_foreach (GTK_TREE_MODEL (priv->method_store), set_method, &info);
 
 	/* Addresses */
-	store = gtk_list_store_new (2, G_TYPE_STRING, G_TYPE_STRING);
+	store = gtk_list_store_new (3, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING);
 	for (i = 0; i < nm_setting_ip6_config_get_num_addresses (setting); i++) {
 		NMIP6Address *addr = nm_setting_ip6_config_get_address (setting, i);
 		const struct in6_addr *tmp_addr;
@@ -326,11 +368,21 @@ populate_ui (CEPageIP6 *self)
 
 		gtk_list_store_append (store, &model_iter);
 
+		/* Address */
 		tmp_addr = nm_ip6_address_get_address (addr);
 		ignored = inet_ntop (AF_INET6, tmp_addr, &buf[0], sizeof (buf));
 		gtk_list_store_set (store, &model_iter, COL_ADDRESS, buf, -1);
+
+		/* Prefix */
 		snprintf (buf, sizeof (buf), "%u", nm_ip6_address_get_prefix (addr));
 		gtk_list_store_set (store, &model_iter, COL_PREFIX, buf, -1);
+
+		/* Gateway */
+		tmp_addr = nm_ip6_address_get_gateway (addr);
+		if (tmp_addr && !IN6_IS_ADDR_UNSPECIFIED (tmp_addr)) {
+			ignored = inet_ntop (AF_INET6, tmp_addr, &buf[0], sizeof (buf));
+			gtk_list_store_set (store, &model_iter, COL_GATEWAY, buf, -1);
+		}
 	}
 
 	gtk_tree_view_set_model (priv->addr_list, GTK_TREE_MODEL (store));
@@ -366,6 +418,10 @@ populate_ui (CEPageIP6 *self)
 	}
 	gtk_entry_set_text (priv->dns_searches, string->str);
 	g_string_free (string, TRUE);
+
+	/* IPv6 required */
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (priv->ip6_required),
+	                              !nm_setting_ip6_config_get_may_fail (setting));
 }
 
 static void
@@ -444,6 +500,37 @@ list_selection_changed (GtkTreeSelection *selection, gpointer user_data)
 		gtk_widget_set_sensitive (button, FALSE);
 }
 
+static void
+cell_editing_canceled (GtkCellRenderer *renderer, gpointer user_data)
+{
+	CEPageIP6 *self;
+	CEPageIP6Private *priv;
+	GtkTreeModel *model = NULL;
+	GtkTreeSelection *selection;
+	GtkTreeIter iter;
+	guint32 column;
+
+	/* user_data disposed? */
+	if (GPOINTER_TO_UINT (g_object_get_data (G_OBJECT (renderer), "ce-page-not-valid")))
+		return;
+
+	self = CE_PAGE_IP6 (user_data);
+	priv = CE_PAGE_IP6_GET_PRIVATE (self);
+
+	if (priv->last_edited) {
+		selection = gtk_tree_view_get_selection (priv->addr_list);
+		if (gtk_tree_selection_get_selected (selection, &model, &iter)) {
+			column = GPOINTER_TO_UINT (g_object_get_data (G_OBJECT (renderer), "column"));
+			gtk_list_store_set (GTK_LIST_STORE (model), &iter, column, priv->last_edited, -1);
+		}
+
+		g_free (priv->last_edited);
+		priv->last_edited = NULL;
+
+		ce_page_changed (CE_PAGE (self));
+	}
+}
+
 static void
 cell_edited (GtkCellRendererText *cell,
              const gchar *path_string,
@@ -458,6 +545,9 @@ cell_edited (GtkCellRendererText *cell,
 	guint32 column;
 	GtkTreeViewColumn *next_col;
 
+	g_free (priv->last_edited);
+	priv->last_edited = NULL;
+
 	column = GPOINTER_TO_UINT (g_object_get_data (G_OBJECT (cell), "column"));
 	gtk_tree_model_get_iter (GTK_TREE_MODEL (store), &iter, path);
 	gtk_list_store_set (store, &iter, column, new_text, -1);
@@ -477,12 +567,22 @@ ip_address_filter_cb (GtkEntry *   entry,
                       const gchar *text,
                       gint         length,
                       gint *       position,
-                      gpointer     data)
+                      gpointer     user_data)
 {
+	CEPageIP6 *self = CE_PAGE_IP6 (user_data);
+	CEPageIP6Private *priv = CE_PAGE_IP6_GET_PRIVATE (self);
 	GtkEditable *editable = GTK_EDITABLE (entry);
-	gboolean numeric = GPOINTER_TO_INT (data);
+	gboolean numeric = FALSE;
 	int i, count = 0;
-	gchar *result = g_new (gchar, length);
+	gchar *result;
+	guint column;
+
+	result = g_malloc0 (length + 1);
+
+	/* The prefix column only allows numbers, no ':' */
+	column = GPOINTER_TO_UINT (g_object_get_data (G_OBJECT (editable), "column"));
+	if (column == COL_PREFIX)
+		numeric = TRUE;
 
 	for (i = 0; i < length; i++) {
 		if ((numeric && g_ascii_isdigit (text[i])) ||
@@ -493,11 +593,13 @@ ip_address_filter_cb (GtkEntry *   entry,
 	if (count > 0) {
 		g_signal_handlers_block_by_func (G_OBJECT (editable),
 		                                 G_CALLBACK (ip_address_filter_cb),
-		                                 data);
+		                                 user_data);
 		gtk_editable_insert_text (editable, result, count, position);
+		g_free (priv->last_edited);
+		priv->last_edited = g_strdup (gtk_editable_get_chars (editable, 0, -1));
 		g_signal_handlers_unblock_by_func (G_OBJECT (editable),
 		                                   G_CALLBACK (ip_address_filter_cb),
-		                                   data);
+		                                   user_data);
 	}
 
 	g_signal_stop_emission_by_name (G_OBJECT (editable), "insert-text");
@@ -508,17 +610,28 @@ static void
 cell_editing_started (GtkCellRenderer *cell,
                       GtkCellEditable *editable,
                       const gchar     *path,
-                      gpointer         data)
+                      gpointer         user_data)
 {
+	CEPageIP6 *self = CE_PAGE_IP6 (user_data);
+	CEPageIP6Private *priv = CE_PAGE_IP6_GET_PRIVATE (self);
+	guint column;
+
 	if (!GTK_IS_ENTRY (editable)) {
 		g_warning ("%s: Unexpected cell editable type.", __func__);
 		return;
 	}
 
+	g_free (priv->last_edited);
+	priv->last_edited = NULL;
+
+	/* Need to pass column # to the editable's insert-text function */	
+	column = GPOINTER_TO_UINT (g_object_get_data (G_OBJECT (cell), "column"));
+	g_object_set_data (G_OBJECT (editable), "column", GUINT_TO_POINTER (column));
+
 	/* Set up the entry filter */
 	g_signal_connect (G_OBJECT (editable), "insert-text",
 	                  (GCallback) ip_address_filter_cb,
-	                  data);
+	                  user_data);
 }
 
 static void
@@ -552,7 +665,7 @@ routes_button_clicked_cb (GtkWidget *button, gpointer user_data)
 	char *tmp;
 
 	toplevel = gtk_widget_get_toplevel (CE_PAGE (self)->page);
-	g_return_if_fail (GTK_WIDGET_TOPLEVEL (toplevel));
+	g_return_if_fail (gtk_widget_is_toplevel (toplevel));
 
 	method = nm_setting_ip6_config_get_method (priv->setting);
 	if (!method || !strcmp (method, NM_SETTING_IP6_CONFIG_METHOD_AUTO))
@@ -604,7 +717,8 @@ finish_setup (CEPageIP6 *self, gpointer unused, GError *error, gpointer user_dat
 	g_object_set (renderer, "editable", TRUE, NULL);
 	g_signal_connect (renderer, "edited", G_CALLBACK (cell_edited), self);
 	g_object_set_data (G_OBJECT (renderer), "column", GUINT_TO_POINTER (COL_ADDRESS));
-	g_signal_connect (renderer, "editing-started", G_CALLBACK (cell_editing_started), GINT_TO_POINTER (FALSE));
+	g_signal_connect (renderer, "editing-started", G_CALLBACK (cell_editing_started), self);
+	g_signal_connect (renderer, "editing-canceled", G_CALLBACK (cell_editing_canceled), self);
 	priv->addr_cells[COL_ADDRESS] = GTK_CELL_RENDERER (renderer);
 
 	offset = gtk_tree_view_insert_column_with_attributes (priv->addr_list,
@@ -620,7 +734,8 @@ finish_setup (CEPageIP6 *self, gpointer unused, GError *error, gpointer user_dat
 	g_object_set (renderer, "editable", TRUE, NULL);
 	g_signal_connect (renderer, "edited", G_CALLBACK (cell_edited), self);
 	g_object_set_data (G_OBJECT (renderer), "column", GUINT_TO_POINTER (COL_PREFIX));
-	g_signal_connect (renderer, "editing-started", G_CALLBACK (cell_editing_started), GINT_TO_POINTER (TRUE));
+	g_signal_connect (renderer, "editing-started", G_CALLBACK (cell_editing_started), self);
+	g_signal_connect (renderer, "editing-canceled", G_CALLBACK (cell_editing_canceled), self);
 	priv->addr_cells[COL_PREFIX] = GTK_CELL_RENDERER (renderer);
 
 	offset = gtk_tree_view_insert_column_with_attributes (priv->addr_list,
@@ -631,6 +746,23 @@ finish_setup (CEPageIP6 *self, gpointer unused, GError *error, gpointer user_dat
 	gtk_tree_view_column_set_expand (GTK_TREE_VIEW_COLUMN (column), TRUE);
 	gtk_tree_view_column_set_clickable (GTK_TREE_VIEW_COLUMN (column), TRUE);
 
+	/* Gateway column */
+	renderer = gtk_cell_renderer_text_new ();
+	g_object_set (renderer, "editable", TRUE, NULL);
+	g_signal_connect (renderer, "edited", G_CALLBACK (cell_edited), self);
+	g_object_set_data (G_OBJECT (renderer), "column", GUINT_TO_POINTER (COL_GATEWAY));
+	g_signal_connect (renderer, "editing-started", G_CALLBACK (cell_editing_started), self);
+	g_signal_connect (renderer, "editing-canceled", G_CALLBACK (cell_editing_canceled), self);
+	priv->addr_cells[COL_GATEWAY] = GTK_CELL_RENDERER (renderer);
+
+	offset = gtk_tree_view_insert_column_with_attributes (priv->addr_list,
+	                                                      -1, _("Gateway"), renderer,
+	                                                      "text", COL_GATEWAY,
+	                                                      NULL);
+	column = gtk_tree_view_get_column (GTK_TREE_VIEW (priv->addr_list), offset - 1);
+	gtk_tree_view_column_set_expand (GTK_TREE_VIEW_COLUMN (column), TRUE);
+	gtk_tree_view_column_set_clickable (GTK_TREE_VIEW_COLUMN (column), TRUE);
+
 	gtk_widget_set_sensitive (GTK_WIDGET (priv->addr_add), TRUE);
 	gtk_widget_set_sensitive (GTK_WIDGET (priv->addr_delete), FALSE);
 
@@ -645,6 +777,8 @@ finish_setup (CEPageIP6 *self, gpointer unused, GError *error, gpointer user_dat
 	method_changed (priv->method, self);
 	g_signal_connect (priv->method, "changed", G_CALLBACK (method_changed), self);
 
+	g_signal_connect_swapped (priv->ip6_required, "toggled", G_CALLBACK (ce_page_changed), self);
+
 	g_signal_connect (priv->routes_button, "clicked", G_CALLBACK (routes_button_clicked_cb), self);
 }
 
@@ -656,32 +790,19 @@ ce_page_ip6_new (NMConnection *connection,
 {
 	CEPageIP6 *self;
 	CEPageIP6Private *priv;
-	CEPage *parent;
 	NMSettingConnection *s_con;
 
-	self = CE_PAGE_IP6 (g_object_new (CE_TYPE_PAGE_IP6,
-	                                  CE_PAGE_CONNECTION, connection,
-	                                  CE_PAGE_PARENT_WINDOW, parent_window,
-	                                  NULL));
-	parent = CE_PAGE (self);
-
-	parent->xml = glade_xml_new (GLADEDIR "/ce-page-ip6.glade", "IP6Page", NULL);
-	if (!parent->xml) {
-		g_set_error (error, 0, 0, "%s", _("Could not load IPv6 user interface."));
-		g_object_unref (self);
+	self = CE_PAGE_IP6 (ce_page_new (CE_TYPE_PAGE_IP6,
+	                                 connection,
+	                                 parent_window,
+	                                 UIDIR "/ce-page-ip6.ui",
+	                                 "IP6Page",
+	                                 _("IPv6 Settings")));
+	if (!self) {
+		g_set_error_literal (error, 0, 0, _("Could not load IPv6 user interface."));
 		return NULL;
 	}
 
-	parent->page = glade_xml_get_widget (parent->xml, "IP6Page");
-	if (!parent->page) {
-		g_set_error (error, 0, 0, "%s", _("Could not load IPv6 user interface."));
-		g_object_unref (self);
-		return NULL;
-	}
-	g_object_ref_sink (parent->page);
-
-	parent->title = g_strdup (_("IPv6 Settings"));
-
 	ip6_private_init (self, connection);
 	priv = CE_PAGE_IP6_GET_PRIVATE (self);
 
@@ -714,6 +835,7 @@ ui_to_setting (CEPageIP6 *self)
 	const char *text;
 	gboolean ignore_auto_dns = FALSE;
 	char **items = NULL, **iter;
+	gboolean may_fail;
 
 	/* Method */
 	if (gtk_combo_box_get_active_iter (priv->method, &tree_iter)) {
@@ -734,6 +856,9 @@ ui_to_setting (CEPageIP6 *self)
 	case IP6_METHOD_SHARED:
 		method = NM_SETTING_IP6_CONFIG_METHOD_SHARED;
 		break;
+	case IP6_METHOD_AUTO_DHCP_ONLY:
+		method = NM_SETTING_IP6_CONFIG_METHOD_DHCP;
+		break;
 	case IP6_METHOD_AUTO_ADDRESSES:
 		ignore_auto_dns = TRUE;
 		/* fall through */
@@ -754,10 +879,12 @@ ui_to_setting (CEPageIP6 *self)
 	iter_valid = gtk_tree_model_get_iter_first (model, &tree_iter);
 	while (iter_valid) {
 		char *item = NULL, *end;
-		struct in6_addr tmp_addr;
+		struct in6_addr tmp_addr, tmp_gw;
+		gboolean have_gw = FALSE;
 		NMIP6Address *addr;
 		guint32 prefix;
 
+		/* IP address */
 		gtk_tree_model_get (model, &tree_iter, COL_ADDRESS, &item, -1);
 		if (!item || !inet_pton (AF_INET6, item, &tmp_addr)) {
 			g_warning ("%s: IPv6 address '%s' missing or invalid!",
@@ -767,6 +894,7 @@ ui_to_setting (CEPageIP6 *self)
 		}
 		g_free (item);
 
+		/* Prefix */
 		gtk_tree_model_get (model, &tree_iter, COL_PREFIX, &item, -1);
 		if (!item) {
 			g_warning ("%s: IPv6 prefix '%s' missing!",
@@ -783,9 +911,25 @@ ui_to_setting (CEPageIP6 *self)
 		}
 		g_free (item);
 
+		/* Gateway */
+		gtk_tree_model_get (model, &tree_iter, COL_GATEWAY, &item, -1);
+		if (item && strlen (item)) {
+			if (!inet_pton (AF_INET6, item, &tmp_gw)) {
+				g_warning ("%s: IPv6 gateway '%s' missing or invalid!",
+				           __func__, item ? item : "<none>");
+				g_free (item);
+				goto out;
+			}
+			if (!IN6_IS_ADDR_UNSPECIFIED (&tmp_gw))
+				have_gw = TRUE;
+		}
+		g_free (item);
+
 		addr = nm_ip6_address_new ();
 		nm_ip6_address_set_address (addr, &tmp_addr);
 		nm_ip6_address_set_prefix (addr, prefix);
+		if (have_gw)
+			nm_ip6_address_set_gateway (addr, &tmp_gw);
 		nm_setting_ip6_config_add_address (priv->setting, addr);
 		nm_ip6_address_unref (addr);
 
@@ -830,6 +974,11 @@ ui_to_setting (CEPageIP6 *self)
 			g_strfreev (items);
 	}
 
+	may_fail = !gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (priv->ip6_required));
+	g_object_set (G_OBJECT (priv->setting),
+	              NM_SETTING_IP6_CONFIG_MAY_FAIL, may_fail,
+	              NULL);
+
 	valid = TRUE;
 
 out:
@@ -859,10 +1008,15 @@ dispose (GObject *object)
 {
 	CEPageIP6 *self = CE_PAGE_IP6 (object);
 	CEPageIP6Private *priv = CE_PAGE_IP6_GET_PRIVATE (self);
+	int i;
 
 	if (priv->window_group)
 		g_object_unref (priv->window_group);
 
+	/* Mark CEPageIP6 object as invalid; store this indication to cells to be usable in callbacks */
+	for (i = 0; i <= COL_LAST; i++)
+		g_object_set_data (G_OBJECT (priv->addr_cells[i]), "ce-page-not-valid", GUINT_TO_POINTER (1));
+
 	g_free (priv->connection_id);
 
 	G_OBJECT_CLASS (ce_page_ip6_parent_class)->dispose (object);
