@@ -17,10 +17,11 @@
  * with this program; if not, write to the Free Software Foundation, Inc.,
  * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
- * (C) Copyright 2008 Red Hat, Inc.
+ * (C) Copyright 2008 - 2010 Red Hat, Inc.
  */
 
 #include <string.h>
+#include <ctype.h>
 
 #include <gtk/gtk.h>
 #include <glib/gi18n.h>
@@ -53,9 +54,8 @@ typedef struct {
 	GtkButton *apn_button;
 	GtkEntry *network_id;
 	GtkComboBox *network_type;
-	GtkComboBox *band;
+	GtkToggleButton *roaming_allowed;
 	GtkEntry *pin;
-	GtkEntry *puk;
 
 	GtkWindowGroup *window_group;
 	gboolean window_added;
@@ -73,51 +73,31 @@ static void
 mobile_private_init (CEPageMobile *self)
 {
 	CEPageMobilePrivate *priv = CE_PAGE_MOBILE_GET_PRIVATE (self);
-	GladeXML *xml;
+	GtkBuilder *builder;
 
-	xml = CE_PAGE (self)->xml;
+	builder = CE_PAGE (self)->builder;
 
-	priv->number = GTK_ENTRY (glade_xml_get_widget (xml, "mobile_number"));
-	priv->username = GTK_ENTRY (glade_xml_get_widget (xml, "mobile_username"));
-	priv->password = GTK_ENTRY (glade_xml_get_widget (xml, "mobile_password"));
+	priv->number = GTK_ENTRY (GTK_WIDGET (gtk_builder_get_object (builder, "mobile_number")));
+	priv->username = GTK_ENTRY (GTK_WIDGET (gtk_builder_get_object (builder, "mobile_username")));
+	priv->password = GTK_ENTRY (GTK_WIDGET (gtk_builder_get_object (builder, "mobile_password")));
 
-	priv->apn = GTK_ENTRY (glade_xml_get_widget (xml, "mobile_apn"));
-	priv->apn_button = GTK_BUTTON (glade_xml_get_widget (xml, "mobile_apn_button"));
-	priv->network_id = GTK_ENTRY (glade_xml_get_widget (xml, "mobile_network_id"));
-	priv->network_type = GTK_COMBO_BOX (glade_xml_get_widget (xml, "mobile_network_type"));
-	priv->band = GTK_COMBO_BOX (glade_xml_get_widget (xml, "mobile_band"));
+	priv->apn = GTK_ENTRY (GTK_WIDGET (gtk_builder_get_object (builder, "mobile_apn")));
+	priv->apn_button = GTK_BUTTON (GTK_WIDGET (gtk_builder_get_object (builder, "mobile_apn_button")));
+	priv->network_id = GTK_ENTRY (GTK_WIDGET (gtk_builder_get_object (builder, "mobile_network_id")));
+	priv->network_type = GTK_COMBO_BOX (GTK_WIDGET (gtk_builder_get_object (builder, "mobile_network_type")));
+	priv->roaming_allowed = GTK_TOGGLE_BUTTON (GTK_WIDGET (gtk_builder_get_object (builder, "mobile_roaming_allowed")));
 
-	priv->pin = GTK_ENTRY (glade_xml_get_widget (xml, "mobile_pin"));
-	priv->puk = GTK_ENTRY (glade_xml_get_widget (xml, "mobile_puk"));
+	priv->pin = GTK_ENTRY (GTK_WIDGET (gtk_builder_get_object (builder, "mobile_pin")));
 
 	priv->window_group = gtk_window_group_new ();
 }
 
-static GHashTable *
-get_secrets (NMConnection *connection, const char *setting_name)
-{
-	GError *error = NULL;
-	GHashTable *secrets;
-
-	secrets = nm_gconf_get_keyring_items (connection,
-	                                      setting_name,
-	                                      FALSE,
-	                                      &error);
-	if (!secrets && error)
-		g_error_free (error);
-
-	return secrets;
-}
-
 static void
 populate_gsm_ui (CEPageMobile *self, NMConnection *connection)
 {
 	CEPageMobilePrivate *priv = CE_PAGE_MOBILE_GET_PRIVATE (self);
 	NMSettingGsm *setting = NM_SETTING_GSM (priv->setting);
 	int type_idx;
-	GHashTable *secrets;
-	GValue *value;
-	GtkWidget *widget;
 	const char *s;
 
 	s = nm_setting_gsm_get_number (setting);
@@ -156,48 +136,16 @@ populate_gsm_ui (CEPageMobile *self, NMConnection *connection)
 	}
 	gtk_combo_box_set_active (priv->network_type, type_idx);
 
-	/* Hide network type widgets; not supported yet */
-	gtk_widget_hide (GTK_WIDGET (priv->network_type));
-	widget = glade_xml_get_widget (CE_PAGE (self)->xml, "type_label");
-	gtk_widget_hide (widget);
-
-	/* Hide Band widgets; not supported yet */
-	widget = glade_xml_get_widget (CE_PAGE (self)->xml, "mobile_band");
-	gtk_widget_hide (widget);
-	widget = glade_xml_get_widget (CE_PAGE (self)->xml, "band_label");
-	gtk_widget_hide (widget);
-
-	secrets = get_secrets (connection, nm_setting_get_name (priv->setting));
+	gtk_toggle_button_set_active (priv->roaming_allowed,
+	                              !nm_setting_gsm_get_home_only (setting));
 
 	s = nm_setting_gsm_get_password (setting);
 	if (s)
 		gtk_entry_set_text (priv->password, s);
-	else if (secrets) {
-		value = g_hash_table_lookup (secrets, NM_SETTING_GSM_PASSWORD);
-		if (value)
-			gtk_entry_set_text (priv->password, g_value_get_string (value));
-	}
 
 	s = nm_setting_gsm_get_pin (setting);
 	if (s)
 		gtk_entry_set_text (priv->pin, s);
-	else if (secrets) {
-		value = g_hash_table_lookup (secrets, NM_SETTING_GSM_PIN);
-		if (value)
-			gtk_entry_set_text (priv->pin, g_value_get_string (value));
-	}
-
-	s = nm_setting_gsm_get_puk (setting);
-	if (s)
-		gtk_entry_set_text (priv->pin, s);
-	else if (secrets) {
-		value = g_hash_table_lookup (secrets, NM_SETTING_GSM_PUK);
-		if (value)
-			gtk_entry_set_text (priv->puk, g_value_get_string (value));
-	}
-
-	if (secrets)
-		g_hash_table_destroy (secrets);
 }
 
 static void
@@ -205,8 +153,6 @@ populate_cdma_ui (CEPageMobile *self, NMConnection *connection)
 {
 	CEPageMobilePrivate *priv = CE_PAGE_MOBILE_GET_PRIVATE (self);
 	NMSettingCdma *setting = NM_SETTING_CDMA (priv->setting);
-	GHashTable *secrets;
-	GValue *value;
 	const char *s;
 
 	s = nm_setting_cdma_get_number (setting);
@@ -217,23 +163,13 @@ populate_cdma_ui (CEPageMobile *self, NMConnection *connection)
 	if (s)
 		gtk_entry_set_text (priv->username, s);
 
-	secrets = get_secrets (connection, nm_setting_get_name (priv->setting));
-
 	s = nm_setting_cdma_get_password (setting);
 	if (s)
 		gtk_entry_set_text (priv->password, s);
-	else if (secrets) {
-		value = g_hash_table_lookup (secrets, NM_SETTING_CDMA_PASSWORD);
-		if (value)
-			gtk_entry_set_text (priv->password, g_value_get_string (value));
-	}
-
-	if (secrets)
-		g_hash_table_destroy (secrets);
 
 	/* Hide GSM specific widgets */
-	gtk_widget_hide (glade_xml_get_widget (CE_PAGE (self)->xml, "mobile_basic_label"));
-	gtk_widget_hide (glade_xml_get_widget (CE_PAGE (self)->xml, "mobile_advanced_vbox"));
+	gtk_widget_hide (GTK_WIDGET (gtk_builder_get_object (CE_PAGE (self)->builder, "mobile_basic_label")));
+	gtk_widget_hide (GTK_WIDGET (gtk_builder_get_object (CE_PAGE (self)->builder, "mobile_advanced_vbox")));
 }
 
 static void
@@ -252,7 +188,6 @@ show_passwords (GtkToggleButton *button, gpointer user_data)
 
 	gtk_entry_set_visibility (priv->password, active);
 	gtk_entry_set_visibility (priv->pin, active);
-	gtk_entry_set_visibility (priv->puk, active);
 }
 
 static void
@@ -297,7 +232,7 @@ apn_button_clicked (GtkButton *button, gpointer user_data)
 	GtkWidget *toplevel;
 
 	toplevel = gtk_widget_get_toplevel (CE_PAGE (self)->page);
-	g_return_if_fail (GTK_WIDGET_TOPLEVEL (toplevel));
+	g_return_if_fail (gtk_widget_is_toplevel (toplevel));
 
 	if (!priv->window_added) {
 		gtk_window_group_add_window (priv->window_group, GTK_WINDOW (toplevel));
@@ -314,6 +249,68 @@ apn_button_clicked (GtkButton *button, gpointer user_data)
 		mobile_wizard_present (wizard);
 }
 
+static void
+network_id_filter_cb (GtkEntry *   entry,
+                      const gchar *text,
+                      gint         length,
+                      gint *       position,
+                      gpointer     user_data)
+{
+	GtkEditable *editable = GTK_EDITABLE (entry);
+	int i, count = 0;
+	gchar *result;
+
+	result = g_malloc0 (length + 1);
+
+	for (i = 0; i < length; i++) {
+		if (isdigit (text[i]))
+			result[count++] = text[i];
+	}
+
+	if (count > 0) {
+		g_signal_handlers_block_by_func (G_OBJECT (editable),
+		                                 G_CALLBACK (network_id_filter_cb),
+		                                 user_data);
+		gtk_editable_insert_text (editable, result, count, position);
+		g_signal_handlers_unblock_by_func (G_OBJECT (editable),
+		                                   G_CALLBACK (network_id_filter_cb),
+		                                   user_data);
+	}
+
+	g_signal_stop_emission_by_name (G_OBJECT (editable), "insert-text");
+	g_free (result);
+}
+
+static void
+apn_filter_cb (GtkEntry *   entry,
+               const gchar *text,
+               gint         length,
+               gint *       position,
+               gpointer     user_data)
+{
+	GtkEditable *editable = GTK_EDITABLE (entry);
+	int i, count = 0;
+	gchar *result = g_new0 (gchar, length);
+
+	for (i = 0; i < length; i++) {
+		if (isalnum (text[i]) || (text[i] == '.') || (text[i] == '-'))
+			result[count++] = text[i];
+	}
+
+	if (count > 0) {
+		g_signal_handlers_block_by_func (G_OBJECT (editable),
+		                                 G_CALLBACK (apn_filter_cb),
+		                                 user_data);
+		gtk_editable_insert_text (editable, result, count, position);
+		g_signal_handlers_unblock_by_func (G_OBJECT (editable),
+		                                   G_CALLBACK (apn_filter_cb),
+		                                   user_data);
+	}
+
+	g_signal_stop_emission_by_name (G_OBJECT (editable), "insert-text");
+	g_free (result);
+}
+
 static void
 finish_setup (CEPageMobile *self, gpointer unused, GError *error, gpointer user_data)
 {
@@ -333,57 +330,56 @@ finish_setup (CEPageMobile *self, gpointer unused, GError *error, gpointer user_
 	g_signal_connect (priv->number, "changed", G_CALLBACK (stuff_changed), self);
 	g_signal_connect (priv->username, "changed", G_CALLBACK (stuff_changed), self);
 	g_signal_connect (priv->password, "changed", G_CALLBACK (stuff_changed), self);
+
 	g_signal_connect (priv->apn, "changed", G_CALLBACK (stuff_changed), self);
+	gtk_entry_set_max_length (priv->apn, 64);  /* APNs are max 64 chars */
+	g_signal_connect (priv->apn, "insert-text", G_CALLBACK (apn_filter_cb), self);
 	g_signal_connect (priv->apn_button, "clicked", G_CALLBACK (apn_button_clicked), self);
+
 	g_signal_connect (priv->network_id, "changed", G_CALLBACK (stuff_changed), self);
+	gtk_entry_set_max_length (priv->network_id, 6);  /* MCC/MNCs are max 6 chars */
+	g_signal_connect (priv->network_id, "insert-text", G_CALLBACK (network_id_filter_cb), self);
+
 	g_signal_connect (priv->network_type, "changed", G_CALLBACK (stuff_changed), self);
 	g_signal_connect (priv->pin, "changed", G_CALLBACK (stuff_changed), self);
-	g_signal_connect (priv->puk, "changed", G_CALLBACK (stuff_changed), self);
+	g_signal_connect (priv->roaming_allowed, "toggled", G_CALLBACK (stuff_changed), self);
 
-	g_signal_connect (glade_xml_get_widget (parent->xml, "mobile_show_passwords"),
+	g_signal_connect (GTK_WIDGET (gtk_builder_get_object (parent->builder, "mobile_show_passwords")),
 	                  "toggled",
 	                  G_CALLBACK (show_passwords),
 	                  self);
 }
 
 CEPage *
-ce_page_mobile_new (NMConnection *connection, GtkWindow *parent_window, GError **error)
+ce_page_mobile_new (NMConnection *connection,
+                    GtkWindow *parent_window,
+                    const char **out_secrets_setting_name,
+                    GError **error)
 {
 	CEPageMobile *self;
 	CEPageMobilePrivate *priv;
-	CEPage *parent;
-	const char *setting_name = NM_SETTING_GSM_SETTING_NAME;
 
-	self = CE_PAGE_MOBILE (g_object_new (CE_TYPE_PAGE_MOBILE,
-	                                     CE_PAGE_CONNECTION, connection,
-	                                     CE_PAGE_PARENT_WINDOW, parent_window,
-	                                     NULL));
-	parent = CE_PAGE (self);
-
-	parent->xml = glade_xml_new (GLADEDIR "/ce-page-mobile.glade", "MobilePage", NULL);
-	if (!parent->xml) {
-		g_set_error (error, 0, 0, "%s", _("Could not load mobile broadband user interface."));
-		g_object_unref (self);
+	self = CE_PAGE_MOBILE (ce_page_new (CE_TYPE_PAGE_MOBILE,
+	                                    connection,
+	                                    parent_window,
+	                                    UIDIR "/ce-page-mobile.ui",
+	                                    "MobilePage",
+	                                    _("Mobile Broadband")));
+	if (!self) {
+		g_set_error_literal (error, 0, 0, _("Could not load mobile broadband user interface."));
 		return NULL;
 	}
 
-	parent->page = glade_xml_get_widget (parent->xml, "MobilePage");
-	if (!parent->page) {
-		g_set_error (error, 0, 0, "%s", _("Could not load mobile broadband user interface."));
-		g_object_unref (self);
-		return NULL;
-	}
-	g_object_ref_sink (parent->page);
-
-	parent->title = g_strdup (_("Mobile Broadband"));
-
 	mobile_private_init (self);
 	priv = CE_PAGE_MOBILE_GET_PRIVATE (self);
 
 	priv->setting = nm_connection_get_setting (connection, NM_TYPE_SETTING_GSM);
-	if (!priv->setting) {
+	if (priv->setting)
+		*out_secrets_setting_name = NM_SETTING_GSM_SETTING_NAME;
+	else {
 		priv->setting = nm_connection_get_setting (connection, NM_TYPE_SETTING_CDMA);
-		setting_name = NM_SETTING_CDMA_SETTING_NAME;
+		if (priv->setting)
+			*out_secrets_setting_name = NM_SETTING_CDMA_SETTING_NAME;
 	}
 
 	if (!priv->setting) {
@@ -393,10 +389,6 @@ ce_page_mobile_new (NMConnection *connection, GtkWindow *parent_window, GError *
 	}
 
 	g_signal_connect (self, "initialized", G_CALLBACK (finish_setup), NULL);
-	if (!ce_page_initialize (parent, setting_name, error)) {
-		g_object_unref (self);
-		return NULL;
-	}
 
 	return CE_PAGE (self);
 }
@@ -418,6 +410,7 @@ gsm_ui_to_setting (CEPageMobile *self)
 {
 	CEPageMobilePrivate *priv = CE_PAGE_MOBILE_GET_PRIVATE (self);
 	int net_type;
+	gboolean roaming_allowed;
 
 	switch (gtk_combo_box_get_active (priv->network_type)) {
 	case NET_TYPE_3G:
@@ -438,15 +431,17 @@ gsm_ui_to_setting (CEPageMobile *self)
 		break;
 	}
 
+	roaming_allowed = gtk_toggle_button_get_active (priv->roaming_allowed);
+
 	g_object_set (priv->setting,
-				  NM_SETTING_GSM_NUMBER,   nm_entry_get_text (priv->number),
-				  NM_SETTING_GSM_USERNAME, nm_entry_get_text (priv->username),
-				  NM_SETTING_GSM_PASSWORD, nm_entry_get_text (priv->password),
-				  NM_SETTING_GSM_APN, nm_entry_get_text (priv->apn),
-				  NM_SETTING_GSM_NETWORK_ID, nm_entry_get_text (priv->network_id),
+				  NM_SETTING_GSM_NUMBER,       nm_entry_get_text (priv->number),
+				  NM_SETTING_GSM_USERNAME,     nm_entry_get_text (priv->username),
+				  NM_SETTING_GSM_PASSWORD,     nm_entry_get_text (priv->password),
+				  NM_SETTING_GSM_APN,          nm_entry_get_text (priv->apn),
+				  NM_SETTING_GSM_NETWORK_ID,   nm_entry_get_text (priv->network_id),
 				  NM_SETTING_GSM_NETWORK_TYPE, net_type,
-				  NM_SETTING_GSM_PIN, nm_entry_get_text (priv->pin),
-				  NM_SETTING_GSM_PUK, nm_entry_get_text (priv->puk),
+				  NM_SETTING_GSM_PIN,          nm_entry_get_text (priv->pin),
+				  NM_SETTING_GSM_HOME_ONLY,    !roaming_allowed,
 				  NULL);
 }
 
@@ -665,8 +660,10 @@ mobile_connection_new (GtkWindow *parent,
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (gsm_radio), TRUE);
 	gtk_box_pack_start (GTK_BOX (vbox), gsm_radio, FALSE, FALSE, 6);
 
+	/* Translators: CDMA has 'D' accelerator key; 'C' collides with 'Cancel' button.
+	                You may need to change it according to your language. */
 	cdma_radio = gtk_radio_button_new_with_mnemonic_from_widget (GTK_RADIO_BUTTON (gsm_radio),
-                                           _("My provider uses _CDMA-based technology (i.e. 1xRTT, EVDO)"));
+                                           _("My provider uses C_DMA-based technology (i.e. 1xRTT, EVDO)"));
 	gtk_box_pack_start (GTK_BOX (vbox), cdma_radio, FALSE, FALSE, 6);
 
 	gtk_widget_show_all (dialog);
