@@ -18,7 +18,7 @@
  * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
  * (C) Copyright 2008 Novell, Inc.
- * (C) Copyright 2008 Red Hat, Inc.
+ * (C) Copyright 2008 - 2010 Red Hat, Inc.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -56,97 +56,73 @@ dialog_set_network_name (NMConnection *connection, GtkEntry *entry)
 
 static WirelessSecurity *
 dialog_set_security (NMConnection *connection,
-					 const char *glade_file,
-					 GtkBox *box)
+                     GtkBuilder *builder,
+                     GtkBox *box)
 {
 	GList *children;
 	GList *iter;
 	WirelessSecurity *security;
 
-	security = (WirelessSecurity *) ws_wpa_eap_new (glade_file, connection, FALSE);
+	security = (WirelessSecurity *) ws_wpa_eap_new (connection, FALSE);
 
 	/* Remove any previous wireless security widgets */
 	children = gtk_container_get_children (GTK_CONTAINER (box));
 	for (iter = children; iter; iter = iter->next)
 		gtk_container_remove (GTK_CONTAINER (box), GTK_WIDGET (iter->data));
+	g_list_free (children);
 
 	gtk_box_pack_start (box, wireless_security_get_widget (security), TRUE, TRUE, 0);
 
 	return security;
 }
 
-static gboolean
-dialog_init (GtkWidget *dialog,
-			 GladeXML *xml,
-			 NMClient *nm_client,
-			 const char *glade_file,
-			 NMConnection *connection)
+GtkWidget *
+nma_wired_dialog_new (NMClient *nm_client,
+                      NMSettingsConnectionInterface *connection,
+                      NMDevice *device)
 {
+	GtkBuilder *builder;
+	GtkWidget *dialog;
+	GError *error = NULL;
 	WirelessSecurity *security;
-	GtkWidget *widget;
 
-	/* Hide bunch of wireless specific widgets */
-	gtk_widget_hide (glade_xml_get_widget (xml, "device_label"));
-	gtk_widget_hide (glade_xml_get_widget (xml, "device_combo"));
-	gtk_widget_hide (glade_xml_get_widget (xml, "security_combo_label"));
-	gtk_widget_hide (glade_xml_get_widget (xml, "security_combo"));
+	builder = gtk_builder_new ();
 
-	/* The dialog won't ever get called for more than one connection for wired */
-	gtk_widget_hide (glade_xml_get_widget (xml, "connection_label"));
-	gtk_widget_hide (glade_xml_get_widget (xml, "connection_combo"));
+	if (!gtk_builder_add_from_file (builder, UIDIR "/wired-8021x.ui", &error)) {
+		g_warning ("Couldn't load builder file: %s", error->message);
+		g_error_free (error);
+		applet_warning_dialog_show (_("The NetworkManager Applet could not find some required resources (the .ui file was not found)."));
+		g_object_unref (builder);
+		return NULL;
+	}
 
-	gtk_window_set_title (GTK_WINDOW (glade_xml_get_widget (xml, "wireless_dialog")),
-	                      _("Wired 802.1X authentication"));
+	dialog = (GtkWidget *) gtk_builder_get_object (builder, "wired_8021x_dialog");
+	if (!dialog) {
+		nm_warning ("Couldn't find wired_8021x_dialog widget.");
+		applet_warning_dialog_show (_("The NetworkManager Applet could not find some required resources (the .ui file was not found)."));
+		g_object_unref (builder);
+		return NULL;
+	}
 
-	dialog_set_network_name (connection, GTK_ENTRY (glade_xml_get_widget (xml, "network_name_entry")));
-	security = dialog_set_security (connection, glade_file, GTK_BOX (glade_xml_get_widget (xml, "security_vbox")));
-	wireless_security_set_changed_notify (security, stuff_changed_cb, glade_xml_get_widget (xml, "ok_button"));
+	gtk_window_set_title (GTK_WINDOW (dialog), _("Wired 802.1X authentication"));
+	gtk_window_set_icon_name (GTK_WINDOW (dialog), "dialog-password");
+	dialog_set_network_name (NM_CONNECTION (connection), GTK_ENTRY (gtk_builder_get_object (builder, "network_name_entry")));
 
+	security = dialog_set_security (NM_CONNECTION (connection), builder, GTK_BOX (gtk_builder_get_object (builder, "security_vbox")));
+	wireless_security_set_changed_notify (security, stuff_changed_cb, GTK_WIDGET (gtk_builder_get_object (builder, "ok_button")));
 	g_object_set_data_full (G_OBJECT (dialog),
 	                        "security", security,
 	                        (GDestroyNotify) wireless_security_unref);
 
-	gtk_window_set_icon_name (GTK_WINDOW (dialog), "dialog-password");
-	widget = glade_xml_get_widget (xml, "image1");
-	gtk_image_set_from_icon_name (GTK_IMAGE (widget), "dialog-password", GTK_ICON_SIZE_DIALOG);
-
-	return TRUE;
-}
-
-GtkWidget *
-nma_wired_dialog_new (const char *glade_file,
-					  NMClient *nm_client,
-					  NMSettingsConnectionInterface *connection,
-					  NMDevice *device)
-{
-	GladeXML *xml;
-	GtkWidget *dialog;
-	gboolean success;
-
-	xml = glade_xml_new (glade_file, "wireless_dialog", NULL);
-	if (!xml) {
-		applet_warning_dialog_show (_("The NetworkManager Applet could not find some required resources (the glade file was not found)."));
-		return NULL;
-	}
-
-	dialog = glade_xml_get_widget (xml, "wireless_dialog");
-	if (!dialog) {
-		nm_warning ("Couldn't find glade wireless_dialog widget.");
-		g_object_unref (xml);
-		return NULL;
-	}
-
-	success = dialog_init (dialog, xml, nm_client, glade_file, NM_CONNECTION (connection));
-	if (!success) {
-		nm_warning ("Couldn't create wired security dialog.");
-		gtk_widget_destroy (dialog);
-		return NULL;
-	}
-
 	g_object_set_data_full (G_OBJECT (dialog),
 	                        "connection", g_object_ref (connection),
 	                        (GDestroyNotify) g_object_unref);
 
+	/* Ensure the builder gets destroyed when the dialog goes away */
+	g_object_set_data_full (G_OBJECT (dialog),
+	                        "builder", builder,
+	                        (GDestroyNotify) g_object_unref);
+
 	return dialog;
 }
 					  
