@@ -17,20 +17,17 @@
  * with this program; if not, write to the Free Software Foundation, Inc.,
  * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
- * (C) Copyright 2007 - 2009 Red Hat, Inc.
+ * (C) Copyright 2007 - 2010 Red Hat, Inc.
  */
 
 #ifndef WS_WPA_EAP_H
 #define WS_WPA_EAP_H
 
-typedef struct {
-	struct _WirelessSecurity parent;
+#include <nm-connection.h>
 
-	GtkSizeGroup *size_group;
-} WirelessSecurityWPAEAP;
+typedef struct _WirelessSecurityWPAEAP WirelessSecurityWPAEAP;
 
-WirelessSecurityWPAEAP * ws_wpa_eap_new (const char *glade_file,
-                                         NMConnection *connection,
+WirelessSecurityWPAEAP * ws_wpa_eap_new (NMConnection *connection,
                                          gboolean is_editor);
 
 #endif /* WS_WPA_EAP_H */
